package org.apache.maven.scm.provider.accurev.cli;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.IOException;

import org.apache.maven.scm.log.ScmLogger;
import org.codehaus.plexus.util.cli.StreamConsumer;

/**
 *
 */
public class CommandOutputConsumer
    implements StreamConsumer
{

    private final ScmLogger logger;

    private StreamConsumer decorated;

    public CommandOutputConsumer( ScmLogger logger, StreamConsumer decorated )
    {
        this.decorated = decorated;
        this.logger = logger;
    }

    public void consumeLine( String line ) throws IOException
    {
        if ( logger.isDebugEnabled() )
        {
            logger.debug( line );
        }

        if ( decorated != null )
        {
            decorated.consumeLine( line );
        }
    }

    public void waitComplete()
    {
        if ( decorated instanceof XppStreamConsumer )
        {
            ( (XppStreamConsumer) decorated ).waitComplete();
        }

    }

}
