package org.apache.maven.scm.repository;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.maven.scm.ScmException;

import java.util.Collections;
import java.util.List;

/**
 * @author <a href="mailto:trygvis@inamo.no">Trygve Laugst&oslash;l</a>
 * @author Olivier Lamy
 *
 */
public class ScmRepositoryException
    extends ScmException
{
    static final long serialVersionUID = -2191549774722212492L;

    private List<String> validationMessages = Collections.emptyList();

    public ScmRepositoryException( String msg )
    {
        super( msg );
    }

    public ScmRepositoryException( String msg, Throwable cause )
    {
        super( msg, cause );
    }

    public ScmRepositoryException( String msg, List<String> validationMessages )
    {
        super( msg );

        this.validationMessages = validationMessages;
    }

    public List<String> getValidationMessages()
    {
        return validationMessages;
    }
}
