package org.apache.maven.scm.provider.hg.command.inventory;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.maven.scm.ChangeSet;

/**
 * Mercurial changeset
 *
 * @author <a href="mailto:lperez@xebia.fr">Laurent Perez</a>
 *
 */
public class HgChangeSet
    extends ChangeSet
{

    private static final long serialVersionUID = -4556377494055110302L;
    private String branch;

    public HgChangeSet()
    {
        super();
    }

    public HgChangeSet( String branch )
    {
        this.branch = branch;
    }

    public String getBranch()
    {
        return branch;
    }

}
